@@ -46,7 +46,6 @@ typedef enum {
 
 /* the following struct defines the action object data structure
  */
-typedef struct action_s action_t;
 struct action_s {
 	time_t	f_time;		/* used for "max. n messages in m seconds" processing */
 	time_t	tActNow;	/* the current time for an action execution. Initially set to -1 and
@@ -69,7 +68,6 @@ struct action_s {
 	struct modInfo_s *pMod;/* pointer to output module handling this selector */
 	void	*pModData;	/* pointer to module data - content is module-specific */
 	sbool	bRepMsgHasMsg;	/* "message repeated..." has msg fragment in it (0-no, 1-yes) */
-	sbool 	requiresDateCall;/* do we need to do a date call before creating templates? */
 	rsRetVal (*submitToActQ)(action_t *, batch_t *);/* function submit message to action queue */
 	rsRetVal (*qConstruct)(struct queue_s *pThis);
 	enum 	{ ACT_STRING_PASSING = 0, ACT_ARRAY_PASSING = 1, ACT_MSG_PASSING = 2,
