@@ -449,8 +449,6 @@ CODESTARTnewInpInst
 
 	pvals = nvlstGetParams(lst, &inppblk, NULL);
 	if(pvals == NULL) {
-		errmsg.LogError(0, RS_RET_MISSING_CNFPARAMS,
-			        "imfile: required parameter are missing\n");
 		ABORT_FINALIZE(RS_RET_MISSING_CNFPARAMS);
 	}
 
@@ -480,7 +478,7 @@ CODESTARTnewInpInst
 			inst->readMode = pvals[i].val.d.n;
 		} else if(!strcmp(inppblk.descr[i].name, "maxlinesatonce")) {
 			inst->maxLinesAtOnce = pvals[i].val.d.n;
-		} else if(!strcmp(inppblk.descr[i].name, "persistStateInterval")) {
+		} else if(!strcmp(inppblk.descr[i].name, "persiststateinterval")) {
 			inst->iPersistStateInterval = pvals[i].val.d.n;
 		} else if(!strcmp(inppblk.descr[i].name, "maxsubmitatonce")) {
 			inst->nMultiSub = pvals[i].val.d.n;
@@ -596,7 +594,7 @@ CODESTARTactivateCnf
 	for(inst = runModConf->root ; inst != NULL ; inst = inst->next) {
 		addListner(inst);
 	}
-	/* if we could not set up any listners, there is no point in running... */
+	/* if we could not set up any listeners, there is no point in running... */
 	if(iFilPtr == 0) {
 		errmsg.LogError(0, NO_ERRCODE, "imfile: no file monitors could be started, "
 				"input not activated.\n");
