@@ -35,8 +35,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <libestr.h>
-#include <json/json.h>
+#include <json.h>
 #include "conf.h"
 #include "syslogd-types.h"
 #include "template.h"
