@@ -40,7 +40,7 @@
 #include <unistd.h>
 #include <libestr.h>
 #include <libee/libee.h>
-#include <json/json.h>
+#include <json.h>
 #include <liblognorm.h>
 #include "conf.h"
 #include "syslogd-types.h"
@@ -65,16 +65,69 @@ DEF_OMOD_STATIC_DATA
 
 typedef struct _instanceData {
 	sbool bUseRawMsg;	/**< use %rawmsg% instead of %msg% */
+	uchar 	*rulebase;	/**< name of rulebase to use */
 	ln_ctx ctxln;		/**< context to be used for liblognorm */
 	ee_ctx ctxee;		/**< context to be used for libee */
 } instanceData;
 
 typedef struct configSettings_s {
 	uchar *rulebase;		/**< name of normalization rulebase to use */
-	sbool bUseRawMsg;	/**< use %rawmsg% instead of %msg% */
+	int bUseRawMsg;	/**< use %rawmsg% instead of %msg% */
 } configSettings_t;
 static configSettings_t cs;
 
+/* tables for interfacing with the v6 config system */
+/* action (instance) parameters */
+static struct cnfparamdescr actpdescr[] = {
+	{ "rulebase", eCmdHdlrGetWord, 1 },
+	{ "userawmsg", eCmdHdlrBinary, 0 }
+};
+static struct cnfparamblk actpblk =
+	{ CNFPARAMBLK_VERSION,
+	  sizeof(actpdescr)/sizeof(struct cnfparamdescr),
+	  actpdescr
+	};
+
+struct modConfData_s {
+	rsconf_t *pConf;	/* our overall config object */
+};
+
+static modConfData_t *loadModConf = NULL;/* modConf ptr to use for the current load process */
+static modConfData_t *runModConf = NULL;/* modConf ptr to use for the current exec process */
+
+
+/* to be called to build the libee part of the instance ONCE ALL PARAMETERS ARE CORRECT
+ * (and set within pData!).
+ */
+static rsRetVal
+buildInstance(instanceData *pData)
+{
+	DEFiRet;
+	if((pData->ctxee = ee_initCtx()) == NULL) {
+		errmsg.LogError(0, RS_RET_ERR_LIBEE_INIT, "error: could not initialize libee "
+				"ctx, cannot activate action");
+		ABORT_FINALIZE(RS_RET_ERR_LIBEE_INIT);
+	}
+
+	if((pData->ctxln = ln_initCtx()) == NULL) {
+		errmsg.LogError(0, RS_RET_ERR_LIBLOGNORM_INIT, "error: could not initialize "
+				"liblognorm ctx, cannot activate action");
+		ee_exitCtx(pData->ctxee);
+		ABORT_FINALIZE(RS_RET_ERR_LIBLOGNORM_INIT);
+	}
+	ln_setEECtx(pData->ctxln, pData->ctxee);
+	if(ln_loadSamples(pData->ctxln, (char*) pData->rulebase) != 0) {
+		errmsg.LogError(0, RS_RET_NO_RULEBASE, "error: normalization rulebase '%s' "
+				"could not be loaded cannot activate action", cs.rulebase);
+		ee_exitCtx(pData->ctxee);
+		ln_exitCtx(pData->ctxln);
+		ABORT_FINALIZE(RS_RET_ERR_LIBLOGNORM_SAMPDB_LOAD);
+	}
+finalize_it:
+	RETiRet;
+}
+
+
 BEGINinitConfVars		/* (re)set config variables to default values */
 CODESTARTinitConfVars 
 	resetConfigVariables(NULL, NULL);
@@ -86,6 +139,35 @@ CODESTARTcreateInstance
 ENDcreateInstance
 
 
+BEGINbeginCnfLoad
+CODESTARTbeginCnfLoad
+	loadModConf = pModConf;
+	pModConf->pConf = pConf;
+ENDbeginCnfLoad
+
+
+BEGINendCnfLoad
+CODESTARTendCnfLoad
+	loadModConf = NULL; /* done loading */
+	/* free legacy config vars */
+	free(cs.rulebase);
+	cs.rulebase = NULL;
+ENDendCnfLoad
+
+BEGINcheckCnf
+CODESTARTcheckCnf
+ENDcheckCnf
+
+BEGINactivateCnf
+CODESTARTactivateCnf
+	runModConf = pModConf;
+ENDactivateCnf
+
+BEGINfreeCnf
+CODESTARTfreeCnf
+ENDfreeCnf
+
+
 BEGINisCompatibleWithFeature
 CODESTARTisCompatibleWithFeature
 ENDisCompatibleWithFeature
@@ -93,6 +175,7 @@ ENDisCompatibleWithFeature
 
 BEGINfreeInstance
 CODESTARTfreeInstance
+	free(pData->rulebase);
 	ee_exitCtx(pData->ctxee);
 	ln_exitCtx(pData->ctxln);
 ENDfreeInstance
@@ -141,7 +224,7 @@ CODESTARTdoAction
 	es_deleteStr(str);
 
 	/* reformat to our json data struct */
-	// TODO: this is all extremly ineffcient!
+	/* TODO: this is all extremly ineffcient! */
 	ee_fmtEventToJSON(event, &str);
 	cstrJSON = es_str2cstr(str, NULL);
 	dbgprintf("mmnormalize generated: %s\n", cstrJSON);
@@ -156,6 +239,59 @@ CODESTARTdoAction
 ENDdoAction
 
 
+static inline void
+setInstParamDefaults(instanceData *pData)
+{
+	pData->rulebase = NULL;
+	pData->bUseRawMsg = 0;
+}
+
+BEGINnewActInst
+	struct cnfparamvals *pvals;
+	int i;
+	int bDestructPValsOnExit;
+CODESTARTnewActInst
+	DBGPRINTF("newActInst (mmnormalize)\n");
+
+	bDestructPValsOnExit = 0;
+	pvals = nvlstGetParams(lst, &actpblk, NULL);
+	if(pvals == NULL) {
+		errmsg.LogError(0, RS_RET_MISSING_CNFPARAMS, "mmnormalize: error reading "
+				"config parameters");
+		ABORT_FINALIZE(RS_RET_MISSING_CNFPARAMS);
+	}
+	bDestructPValsOnExit = 1;
+
+	if(Debug) {
+		dbgprintf("action param blk in mmnormalize:\n");
+		cnfparamsPrint(&actpblk, pvals);
+	}
+
+	CHKiRet(createInstance(&pData));
+	setInstParamDefaults(pData);
+
+	for(i = 0 ; i < actpblk.nParams ; ++i) {
+		if(!pvals[i].bUsed)
+			continue;
+		if(!strcmp(actpblk.descr[i].name, "rulebase")) {
+			pData->rulebase = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else if(!strcmp(actpblk.descr[i].name, "userawmsg")) {
+			pData->bUseRawMsg = (int) pvals[i].val.d.n;
+		} else {
+			DBGPRINTF("mmnormalize: program error, non-handled "
+			  "param '%s'\n", actpblk.descr[i].name);
+		}
+	}
+	CODE_STD_STRING_REQUESTnewActInst(1)
+	CHKiRet(OMSRsetEntry(*ppOMSR, 0, NULL, OMSR_TPL_AS_MSG));
+
+	iRet = buildInstance(pData);
+CODE_STD_FINALIZERnewActInst
+	if(bDestructPValsOnExit)
+		cnfparamvalsDestruct(pvals, &actpblk);
+ENDnewActInst
+
+
 BEGINparseSelectorAct
 CODESTARTparseSelectorAct
 CODE_STD_STRING_REQUESTparseSelectorAct(1)
@@ -165,15 +301,21 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 	}
 
 	if(cs.rulebase == NULL) {
-		errmsg.LogError(0, RS_RET_NO_RULESET, "error: no normalization rulebase was specified, use "
+		errmsg.LogError(0, RS_RET_NO_RULEBASE, "error: no normalization rulebase was specified, use "
 				"$MMNormalizeSampleDB directive first!");
-		ABORT_FINALIZE(RS_RET_NO_RULESET);
+		ABORT_FINALIZE(RS_RET_NO_RULEBASE);
 	}
 
 	/* ok, if we reach this point, we have something for us */
 	p += sizeof(":mmnormalize:") - 1; /* eat indicator sequence  (-1 because of '\0'!) */
 	CHKiRet(createInstance(&pData));
 
+	pData->rulebase = cs.rulebase;
+	pData->bUseRawMsg = cs.bUseRawMsg;
+	/* all config vars auto-reset! */
+	cs.bUseRawMsg = 0;
+	cs.rulebase = NULL; /* we used it up! */
+
 	/* check if a non-standard template is to be applied */
 	if(*(p-1) == ';')
 		--p;
@@ -181,34 +323,7 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 	 * the format specified (if any) is always ignored.
 	 */
 	CHKiRet(cflineParseTemplateName(&p, *ppOMSR, 0, OMSR_TPL_AS_MSG, (uchar*) "RSYSLOG_FileFormat"));
-
-	/* finally build the instance */
-	if((pData->ctxee = ee_initCtx()) == NULL) {
-		errmsg.LogError(0, RS_RET_NO_RULESET, "error: could not initialize libee ctx, cannot "
-				"activate action");
-		ABORT_FINALIZE(RS_RET_ERR_LIBEE_INIT);
-	}
-
-	if((pData->ctxln = ln_initCtx()) == NULL) {
-		errmsg.LogError(0, RS_RET_NO_RULESET, "error: could not initialize liblognorm ctx, cannot "
-				"activate action");
-		ee_exitCtx(pData->ctxee);
-		ABORT_FINALIZE(RS_RET_ERR_LIBLOGNORM_INIT);
-	}
-	ln_setEECtx(pData->ctxln, pData->ctxee);
-	if(ln_loadSamples(pData->ctxln, (char*) cs.rulebase) != 0) {
-		errmsg.LogError(0, RS_RET_NO_RULESET, "error: normalization rulebase '%s' could not be loaded "
-				"cannot activate action", cs.rulebase);
-		ee_exitCtx(pData->ctxee);
-		ln_exitCtx(pData->ctxln);
-		ABORT_FINALIZE(RS_RET_ERR_LIBLOGNORM_SAMPDB_LOAD);
-	}
-	pData->bUseRawMsg = cs.bUseRawMsg;
-
-	/* all config vars auto-reset! */
-	cs.bUseRawMsg = 0;
-	free(cs.rulebase);
-	cs.rulebase = NULL;
+	CHKiRet(buildInstance(pData));
 CODE_STD_FINALIZERparseSelectorAct
 ENDparseSelectorAct
 
@@ -222,6 +337,8 @@ ENDmodExit
 BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
+CODEqueryEtryPt_STD_CONF2_QUERIES
+CODEqueryEtryPt_STD_CONF2_OMOD_QUERIES
 ENDqueryEtryPt
 
 
@@ -256,6 +373,7 @@ INITLegCnfVars
 	*ipIFVersProvided = CURR_MOD_IF_VERSION;
 		/* we only support the current interface specification */
 CODEmodInit_QueryRegCFSLineHdlr
+	DBGPRINTF("mmnormalize: module compiled with rsyslog version %s.\n", VERSION);
 	/* check if the rsyslog core supports parameter passing code */
 	bMsgPassingSupported = 0;
 	localRet = pHostQueryEtryPt((uchar*)"OMSRgetSupportedTplOpts",
@@ -279,7 +397,7 @@ CODEmodInit_QueryRegCFSLineHdlr
 	
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"mmnormalizerulebase", 0, eCmdHdlrGetWord,
 				    setRuleBase, NULL, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"mmnormalizeuserawmsg", 0, eCmdHdlrInt,
+	CHKiRet(omsdRegCFSLineHdlr((uchar *)"mmnormalizeuserawmsg", 0, eCmdHdlrBinary,
 				    NULL, &cs.bUseRawMsg, STD_LOADABLE_MODULE_ID));
 	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler,
 				    resetConfigVariables, NULL, STD_LOADABLE_MODULE_ID));
