@@ -11,11 +11,11 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *       http://www.apache.org/licenses/LICENSE-2.0
  *       -or-
  *       see COPYING.ASL20 in the source distribution
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -79,12 +79,14 @@ typedef struct _instanceData {
 	uchar *parent;
 	uchar *tplName;
 	uchar *timeout;
+	uchar *bulkId;
 	uchar *restURL;		/* last used URL for error reporting */
 	uchar *errorFile;
 	char *reply;
 	sbool dynSrchIdx;
 	sbool dynSrchType;
 	sbool dynParent;
+	sbool dynBulkId;
 	sbool bulkmode;
 	sbool asyncRepl;
 	struct {
@@ -114,7 +116,9 @@ static struct cnfparamdescr actpdescr[] = {
 	{ "asyncrepl", eCmdHdlrBinary, 0 },
 	{ "timeout", eCmdHdlrGetWord, 0 },
 	{ "errorfile", eCmdHdlrGetWord, 0 },
-	{ "template", eCmdHdlrGetWord, 1 }
+	{ "template", eCmdHdlrGetWord, 1 },
+	{ "dynbulkid", eCmdHdlrBinary, 0 },
+	{ "bulkid", eCmdHdlrGetWord, 0 },
 };
 static struct cnfparamblk actpblk =
 	{ CNFPARAMBLK_VERSION,
@@ -156,6 +160,7 @@ CODESTARTfreeInstance
 	free(pData->timeout);
 	free(pData->restURL);
 	free(pData->errorFile);
+	free(pData->bulkId);
 ENDfreeInstance
 
 BEGINdbgPrintInstInfo
@@ -177,6 +182,8 @@ CODESTARTdbgPrintInstInfo
 	dbgprintf("\tbulkmode=%d\n", pData->bulkmode);
 	dbgprintf("\terrorfile='%s'\n", pData->errorFile == NULL ?
 		(uchar*)"(not configured)" : pData->errorFile);
+	dbgprintf("\tdynbulkid=%d\n", pData->dynBulkId);
+	dbgprintf("\tbulkid='%s'\n", pData->bulkId);
 ENDdbgPrintInstInfo
 
 
@@ -217,10 +224,16 @@ checkConn(instanceData *pData)
 		DBGPRINTF("omelasticsearch: checkConn() curl_easy_init() failed\n");
 		ABORT_FINALIZE(RS_RET_SUSPENDED);
 	}
+	/* Bodypart of request not needed, so set curl opt to nobody and httpget, otherwise lib-curl could sigsegv */
+	curl_easy_setopt(curl, CURLOPT_HTTPGET, TRUE);
+	curl_easy_setopt(curl, CURLOPT_NOBODY, TRUE); 
+	/* Only enable for debugging 
+	curl_easy_setopt(curl, CURLOPT_VERBOSE, TRUE); */
+
 	cstr = es_str2cstr(url, NULL);
 	curl_easy_setopt(curl, CURLOPT_URL, cstr);
 	free(cstr);
- 
+
 	pData->reply = NULL;
 	pData->replyLen = 0;
 	curl_easy_setopt(curl, CURLOPT_WRITEDATA, pData);
@@ -250,7 +263,8 @@ ENDtryResume
 /* get the current index and type for this message */
 static inline void
 getIndexTypeAndParent(instanceData *pData, uchar **tpls,
-		      uchar **srchIndex, uchar **srchType, uchar **parent)
+		      uchar **srchIndex, uchar **srchType, uchar **parent,
+			  uchar **bulkId)
 {
 	if(pData->dynSrchIdx) {
 		*srchIndex = tpls[1];
@@ -258,15 +272,27 @@ getIndexTypeAndParent(instanceData *pData, uchar **tpls,
 			*srchType = tpls[2];
 			if(pData->dynParent) {
 				*parent = tpls[3];
+				if(pData->dynBulkId) {
+					*bulkId = tpls[4];
+				}
 			} else {
 				*parent = pData->parent;
+				if(pData->dynBulkId) {
+					*bulkId = tpls[3];
+				}
 			}
 		} else  {
 			*srchType = pData->searchType;
 			if(pData->dynParent) {
 				*parent = tpls[2];
+				if(pData->dynBulkId) {
+					*bulkId = tpls[3];
+				}
 			} else {
 				*parent = pData->parent;
+				if(pData->dynBulkId) {
+					*bulkId = tpls[2];
+				}
 			}
 		}
 	} else {
@@ -275,15 +301,27 @@ getIndexTypeAndParent(instanceData *pData, uchar **tpls,
 			*srchType = tpls[1];
 			if(pData->dynParent) {
 				*parent = tpls[2];
+                 if(pData->dynBulkId) {
+                    *bulkId = tpls[3];
+                }
 			} else {
 				*parent = pData->parent;
+                if(pData->dynBulkId) {
+                    *bulkId = tpls[2];
+                }
 			}
 		} else  {
 			*srchType = pData->searchType;
 			if(pData->dynParent) {
 				*parent = tpls[1];
+                if(pData->dynBulkId) {
+                    *bulkId = tpls[2];
+                }
 			} else {
 				*parent = pData->parent;
+                if(pData->dynBulkId) {
+                    *bulkId = tpls[1];
+                }
 			}
 		}
 	}
@@ -297,6 +335,7 @@ setCurlURL(instanceData *pData, uchar **tpls)
 	uchar *searchIndex;
 	uchar *searchType;
 	uchar *parent;
+	uchar *bulkId;
 	es_str_t *url;
 	int rLocal;
 	int r;
@@ -308,7 +347,7 @@ setCurlURL(instanceData *pData, uchar **tpls)
 		r = es_addBuf(&url, "_bulk", sizeof("_bulk")-1);
 		parent = NULL;
 	} else {
-		getIndexTypeAndParent(pData, tpls, &searchIndex, &searchType, &parent);
+		getIndexTypeAndParent(pData, tpls, &searchIndex, &searchType, &parent, &bulkId);
 		r = es_addBuf(&url, (char*)searchIndex, ustrlen(searchIndex));
 		if(r == 0) r = es_addChar(&url, '/');
 		if(r == 0) r = es_addBuf(&url, (char*)searchType, ustrlen(searchType));
@@ -330,20 +369,20 @@ setCurlURL(instanceData *pData, uchar **tpls)
 
 	free(pData->restURL);
 	pData->restURL = (uchar*)es_str2cstr(url, NULL);
-	curl_easy_setopt(pData->curlHandle, CURLOPT_URL, pData->restURL); 
+	curl_easy_setopt(pData->curlHandle, CURLOPT_URL, pData->restURL);
 	es_deleteStr(url);
 	DBGPRINTF("omelasticsearch: using REST URL: '%s'\n", pData->restURL);
 
 	if(pData->uid != NULL) {
 		rLocal = snprintf(authBuf, sizeof(authBuf), "%s:%s", pData->uid,
 			         (pData->pwd == NULL) ? "" : (char*)pData->pwd);
-		if(rLocal != (int) es_strlen(url)) {
+		if(rLocal < 1) {
 			errmsg.LogError(0, RS_RET_ERR, "omelasticsearch: snprintf failed "
 				"when trying to build auth string (return %d)\n",
 				rLocal);
 			ABORT_FINALIZE(RS_RET_ERR);
 		}
-		curl_easy_setopt(pData->curlHandle, CURLOPT_USERPWD, authBuf); 
+		curl_easy_setopt(pData->curlHandle, CURLOPT_USERPWD, authBuf);
 		curl_easy_setopt(pData->curlHandle, CURLOPT_PROXYAUTH, CURLAUTH_ANY);
 	}
 finalize_it:
@@ -363,13 +402,15 @@ buildBatch(instanceData *pData, uchar *message, uchar **tpls)
 	uchar *searchIndex;
 	uchar *searchType;
 	uchar *parent;
+	uchar *bulkId = NULL;
 	DEFiRet;
 #	define META_STRT "{\"index\":{\"_index\": \""
 #	define META_TYPE "\",\"_type\":\""
 #	define META_PARENT "\",\"_parent\":\""
+#   define META_ID "\", \"_id\":\""
 #	define META_END  "\"}}\n"
 
-	getIndexTypeAndParent(pData, tpls, &searchIndex, &searchType, &parent);
+	getIndexTypeAndParent(pData, tpls, &searchIndex, &searchType, &parent, &bulkId);
 	r = es_addBuf(&pData->batch.data, META_STRT, sizeof(META_STRT)-1);
 	if(r == 0) r = es_addBuf(&pData->batch.data, (char*)searchIndex,
 				 ustrlen(searchIndex));
@@ -380,6 +421,10 @@ buildBatch(instanceData *pData, uchar *message, uchar **tpls)
 		if(r == 0) r = es_addBuf(&pData->batch.data, META_PARENT, sizeof(META_PARENT)-1);
 		if(r == 0) r = es_addBuf(&pData->batch.data, (char*)parent, ustrlen(parent));
 	}
+	if(bulkId != NULL) {
+		if(r == 0) r = es_addBuf(&pData->batch.data, META_ID, sizeof(META_ID)-1);
+		if(r == 0) r = es_addBuf(&pData->batch.data, (char*)bulkId, ustrlen(bulkId));
+	}
 	if(r == 0) r = es_addBuf(&pData->batch.data, META_END, sizeof(META_END)-1);
 	if(r == 0) r = es_addBuf(&pData->batch.data, (char*)message, length);
 	if(r == 0) r = es_addBuf(&pData->batch.data, "\n", sizeof("\n")-1);
@@ -405,9 +450,11 @@ writeDataError(instanceData *pData, cJSON **pReplyRoot, uchar *reqmsg)
 	cJSON *errRoot;
 	cJSON *req;
 	cJSON *replyRoot = *pReplyRoot;
+	size_t toWrite;
+	ssize_t wrRet;
 	char errStr[1024];
 	DEFiRet;
-	
+
 	if(pData->errorFile == NULL) {
 		DBGPRINTF("omelasticsearch: no local error logger defined - "
 		          "ignoring ES error information\n");
@@ -432,15 +479,21 @@ writeDataError(instanceData *pData, cJSON **pReplyRoot, uchar *reqmsg)
 	cJSON_AddItemToObject(errRoot, "request", req);
 	cJSON_AddItemToObject(errRoot, "reply", replyRoot);
 	rendered = cJSON_Print(errRoot);
-DBGPRINTF("omelasticsearch: error record: '%s'\n", rendered);
-	write(pData->fdErrFile, rendered, strlen(rendered));
-	free(rendered);
+	/* we do not do real error-handling on the err file, as this finally complicates
+	 * things way to much.
+	 */
+	DBGPRINTF("omelasticsearch: error record: '%s'\n", rendered);
+	toWrite = strlen(rendered);
+	wrRet = write(pData->fdErrFile, rendered, toWrite);
+	if(wrRet != (ssize_t) toWrite) {
+		DBGPRINTF("omelasticsearch: error %d writing error file, write returns %lld\n",
+			  errno, (long long) wrRet);
+	}
 	cJSON_Delete(errRoot);
 	*pReplyRoot = NULL; /* tell caller not to delete once again! */
 
 finalize_it:
-	if(rendered != NULL)
-		free(rendered);
+	free(rendered);
 	RETiRet;
 }
 
@@ -514,7 +567,7 @@ checkResult(instanceData *pData, uchar *reqmsg)
 	}
 
 	/* Note: we ignore errors writing the error file, as we cannot handle
-	 * these in any case. 
+	 * these in any case.
 	 */
 	if(iRet == RS_RET_DATAFAIL) {
 		writeDataError(pData, &root, reqmsg);
@@ -542,8 +595,8 @@ curlPost(instanceData *pData, uchar *message, int msglen, uchar **tpls)
 		CHKiRet(setCurlURL(pData, tpls));
 
 	curl_easy_setopt(curl, CURLOPT_WRITEDATA, pData);
-	curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (char *)message); 
-	curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, msglen); 
+	curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (char *)message);
+	curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, msglen);
 	code = curl_easy_perform(curl);
 	switch (code) {
 		case CURLE_COULDNT_RESOLVE_HOST:
@@ -560,8 +613,11 @@ curlPost(instanceData *pData, uchar *message, int msglen, uchar **tpls)
 			break;
 	}
 
-	pData->reply[pData->replyLen] = '\0'; /* byte has been reserved in malloc */
-	DBGPRINTF("omelasticsearch: es reply: '%s'\n", pData->reply);
+	DBGPRINTF("omelasticsearch: pData replyLen = '%d'\n", pData->replyLen);
+	if (pData->replyLen > 0) {
+		pData->reply[pData->replyLen] = '\0'; /* Append 0 Byte if replyLen is above 0 - byte has been reserved in malloc */
+	}
+	DBGPRINTF("omelasticsearch: pData reply: '%s'\n", pData->reply);
 
 	CHKiRet(checkResult(pData, message));
 finalize_it:
@@ -595,12 +651,17 @@ ENDdoAction
 
 
 BEGINendTransaction
-	char *cstr;
+	char *cstr = NULL;
 CODESTARTendTransaction
 dbgprintf("omelasticsearch: endTransaction init\n");
-	cstr = es_str2cstr(pData->batch.data, NULL);
-	dbgprintf("omelasticsearch: endTransaction, batch: '%s'\n", cstr);
-	CHKiRet(curlPost(pData, (uchar*) cstr, strlen(cstr), NULL));
+	/* End Transaction only if batch data is not empty */
+	if (pData->batch.data != NULL ) {
+		cstr = es_str2cstr(pData->batch.data, NULL);
+		dbgprintf("omelasticsearch: endTransaction, batch: '%s'\n", cstr);
+		CHKiRet(curlPost(pData, (uchar*) cstr, strlen(cstr), NULL));
+	}
+	else
+		dbgprintf("omelasticsearch: endTransaction, pData->batch.data is NULL, nothing to send. \n");
 finalize_it:
 	free(cstr);
 dbgprintf("omelasticsearch: endTransaction done with %d\n", iRet);
@@ -639,10 +700,10 @@ curlSetup(instanceData *pData)
 	}
 
 	header = curl_slist_append(NULL, "Content-Type: text/json; charset=utf-8");
-	curl_easy_setopt(handle, CURLOPT_HTTPHEADER, header); 
+	curl_easy_setopt(handle, CURLOPT_HTTPHEADER, header);
 
 	curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, curlResult);
-	curl_easy_setopt(handle, CURLOPT_POST, 1); 
+	curl_easy_setopt(handle, CURLOPT_POST, 1);
 
 	pData->curlHandle = handle;
 	pData->postHeader = header;
@@ -680,6 +741,8 @@ setInstParamDefaults(instanceData *pData)
 	pData->bulkmode = 0;
 	pData->tplName = NULL;
 	pData->errorFile = NULL;
+	pData->dynBulkId= 0;
+	pData->bulkId = NULL;
 }
 
 BEGINnewActInst
@@ -727,12 +790,16 @@ CODESTARTnewActInst
 			pData->asyncRepl = pvals[i].val.d.n;
 		} else if(!strcmp(actpblk.descr[i].name, "template")) {
 			pData->tplName = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else if(!strcmp(actpblk.descr[i].name, "dynbulkid")) {
+			pData->dynBulkId = pvals[i].val.d.n;
+		} else if(!strcmp(actpblk.descr[i].name, "bulkid")) {
+			pData->bulkId = (uchar*)es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else {
 			dbgprintf("omelasticsearch: program error, non-handled "
 			  "param '%s'\n", actpblk.descr[i].name);
 		}
 	}
-	
+
 	if(pData->pwd != NULL && pData->uid == NULL) {
 		errmsg.LogError(0, RS_RET_UID_MISSING,
 			"omelasticsearch: password is provided, but no uid "
@@ -757,6 +824,12 @@ CODESTARTnewActInst
 			"name for parent template given - action definition invalid");
 		ABORT_FINALIZE(RS_RET_CONFIG_ERROR);
 	}
+	if(pData->dynBulkId && pData->bulkId == NULL) {
+		errmsg.LogError(0, RS_RET_CONFIG_ERROR,
+			"omelasticsearch: requested dynamic bulkid, but no "
+			"name for bulkid template given - action definition invalid");
+		ABORT_FINALIZE(RS_RET_CONFIG_ERROR);
+	}
 
 	if(pData->bulkmode) {
 		pData->batch.currTpl1 = NULL;
@@ -772,8 +845,9 @@ CODESTARTnewActInst
 	if(pData->dynSrchIdx) ++iNumTpls;
 	if(pData->dynSrchType) ++iNumTpls;
 	if(pData->dynParent) ++iNumTpls;
+	if(pData->dynBulkId) ++iNumTpls;
 	DBGPRINTF("omelasticsearch: requesting %d templates\n", iNumTpls);
-	CODE_STD_STRING_REQUESTparseSelectorAct(iNumTpls)
+	CODE_STD_STRING_REQUESTnewActInst(iNumTpls)
 
 	CHKiRet(OMSRsetEntry(*ppOMSR, 0, (uchar*)strdup((pData->tplName == NULL) ?
 					    " StdJSONFmt" : (char*)pData->tplName),
@@ -793,11 +867,29 @@ CODESTARTnewActInst
 			if(pData->dynParent) {
 				CHKiRet(OMSRsetEntry(*ppOMSR, 3, ustrdup(pData->parent),
 					OMSR_NO_RQD_TPL_OPTS));
+				if(pData->dynBulkId) {
+					CHKiRet(OMSRsetEntry(*ppOMSR, 4, ustrdup(pData->bulkId),
+						OMSR_NO_RQD_TPL_OPTS));
+				}
+			} else {
+				if(pData->dynBulkId) {
+					CHKiRet(OMSRsetEntry(*ppOMSR, 3, ustrdup(pData->bulkId),
+						OMSR_NO_RQD_TPL_OPTS));
+				}
 			}
 		} else {
 			if(pData->dynParent) {
 				CHKiRet(OMSRsetEntry(*ppOMSR, 2, ustrdup(pData->parent),
 					OMSR_NO_RQD_TPL_OPTS));
+				if(pData->dynBulkId) {
+					CHKiRet(OMSRsetEntry(*ppOMSR, 3, ustrdup(pData->bulkId),
+						OMSR_NO_RQD_TPL_OPTS));
+				}
+			} else {
+				if(pData->dynBulkId) {
+					CHKiRet(OMSRsetEntry(*ppOMSR, 2, ustrdup(pData->bulkId),
+						OMSR_NO_RQD_TPL_OPTS));
+				}
 			}
 		}
 	} else {
@@ -807,12 +899,30 @@ CODESTARTnewActInst
 			if(pData->dynParent) {
 				CHKiRet(OMSRsetEntry(*ppOMSR, 2, ustrdup(pData->parent),
 					OMSR_NO_RQD_TPL_OPTS));
+                if(pData->dynBulkId) {
+                    CHKiRet(OMSRsetEntry(*ppOMSR, 3, ustrdup(pData->bulkId),
+                        OMSR_NO_RQD_TPL_OPTS));
+                }
+			} else {
+				if(pData->dynBulkId) {
+					CHKiRet(OMSRsetEntry(*ppOMSR, 2, ustrdup(pData->bulkId),
+						OMSR_NO_RQD_TPL_OPTS));
+				}
 			}
 		} else {
 			if(pData->dynParent) {
 				CHKiRet(OMSRsetEntry(*ppOMSR, 1, ustrdup(pData->parent),
 					OMSR_NO_RQD_TPL_OPTS));
-			}
+                if(pData->dynBulkId) {
+                    CHKiRet(OMSRsetEntry(*ppOMSR, 2, ustrdup(pData->bulkId),
+                        OMSR_NO_RQD_TPL_OPTS));
+                }
+			} else {
+				if(pData->dynBulkId) {
+					CHKiRet(OMSRsetEntry(*ppOMSR, 1, ustrdup(pData->bulkId),
+						OMSR_NO_RQD_TPL_OPTS));
+				}
+            }
 		}
 	}
 
