@@ -105,6 +105,7 @@ BEGINdoAction
 	int iBuf;
 	char szBuf[65564];
 	size_t len;
+	int r;
 CODESTARTdoAction
 	if(pData->bUseArrayInterface) {
 		/* if we use array passing, we need to put together a string
@@ -136,9 +137,19 @@ CODESTARTdoAction
 		toWrite = (char*) ppString[0];
 	}
 	len = strlen(toWrite);
-	write(1, toWrite, len); /* 1 is stdout! */
+	/* the following if's are just to silence compiler warnings. If someone 
+	 * actually intends to use this module in production (why???), this code
+	 * needs to be more solid. -- rgerhards, 2012-11-28
+	 */
+	if((r = write(1, toWrite, len)) != (int) len) { /* 1 is stdout! */
+		DBGPRINTF("omstdout: error %d writing to stdout[%d]: %s\n",
+			r, len, toWrite);
+	}
 	if(pData->bEnsureLFEnding && toWrite[len-1] != '\n') {
-		write(1, "\n", 1); /* write missing LF */
+		if((r = write(1, "\n", 1)) != 1) { /* write missing LF */
+			DBGPRINTF("omstdout: error %d writing \\n to stdout\n",
+				r);
+		}
 	}
 ENDdoAction
 
@@ -175,6 +186,7 @@ ENDmodExit
 BEGINqueryEtryPt
 CODESTARTqueryEtryPt
 CODEqueryEtryPt_STD_OMOD_QUERIES
+CODEqueryEtryPt_STD_CONF2_CNFNAME_QUERIES 
 ENDqueryEtryPt
 
 
