@@ -113,7 +113,7 @@ static rsRetVal modGetID(void **pID) \
 /* macro to provide the v6 config system module name
  */
 #define MODULE_CNFNAME(name) \
-static __attribute__((unused)) rsRetVal modGetCnfName(uchar **cnfName) \
+static rsRetVal modGetCnfName(uchar **cnfName) \
 	{ \
 		*cnfName = (uchar*) name; \
 		return RS_RET_OK;\
@@ -246,7 +246,8 @@ static rsRetVal dbgPrintInstInfo(void *pModData)\
 	instanceData *pData = NULL;
 
 #define CODESTARTdbgPrintInstInfo \
-	pData = (instanceData*) pModData;
+	pData = (instanceData*) pModData; \
+	(void)pData; /* prevent compiler warning if unused! */
 
 #define ENDdbgPrintInstInfo \
 	RETiRet;\
@@ -937,6 +938,28 @@ static rsRetVal doHUP(instanceData __attribute__((unused)) *pData)\
 }
 
 
+/* SetShutdownImmdtPtr()
+ * This function is optional. If defined by an output plugin, it is called 
+ * each time the action is invoked to set the "ShutdownImmediate" pointer,
+ * which is used during termination to indicate the action should shutdown
+ * as quickly as possible.
+ */
+#define CODEqueryEtryPt_SetShutdownImmdtPtr \
+	else if(!strcmp((char*) name, "SetShutdownImmdtPtr")) {\
+		*pEtryPoint = SetShutdownImmdtPtr;\
+	}
+#define BEGINSetShutdownImmdtPtr \
+static rsRetVal SetShutdownImmdtPtr(instanceData __attribute__((unused)) *pData, int *pPtr)\
+{\
+	DEFiRet;
+
+#define CODESTARTSetShutdownImmdtPtr 
+
+#define ENDSetShutdownImmdtPtr \
+	RETiRet;\
+}
+
+
 /* parse() - main entry point of parser modules
  */
 #define BEGINparse \
