@@ -30,13 +30,11 @@
 
 #include <pthread.h>
 #include <libestr.h>
-#include <json/json.h>
+#include <json.h>
 #include "obj.h"
 #include "syslogd-types.h"
 #include "template.h"
 #include "atomic.h"
-#include "libee/libee.h"
-
 
 /* rgerhards 2004-11-08: The following structure represents a
  * syslog message. 
