@@ -30,7 +30,7 @@ typedef nsd_if_t nsd_ptcp_if_t; /* we just *implement* this interface */
 /* the nsd_ptcp object */
 struct nsd_ptcp_s {
 	BEGINobjInstance; /* Data to implement generic object - MUST be the first data element! */
-	uchar *pRemHostIP; /**< IP address of remote peer (currently used in server mode, only) */
+	prop_t *remoteIP; /**< IP address of remote peer (currently used in server mode, only) */
 	uchar *pRemHostName; /**< host name of remote peer (currently used in server mode, only) */
 	struct sockaddr_storage remAddr; /**< remote addr as sockaddr - used for legacy ACL code */
 	int sock;	/**< the socket we use for regular, single-socket, operations */
