@@ -11,7 +11,7 @@
  *
  * Module begun 2009-06-10 by Rainer Gerhards
  *
- * Copyright 2009-2012 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2009-2013 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
@@ -181,7 +181,7 @@ processBatchMultiRuleset(batch_t *pBatch)
 	do {
 		bHaveUnprocessed = 0;
 		/* search for first unprocessed element */
-		for(iStart = 0 ; iStart < pBatch->nElem && pBatch->pElem[iStart].state == BATCH_STATE_DISC ; ++iStart)
+		for(iStart = 0 ; iStart < pBatch->nElem && pBatch->eltState[iStart] == BATCH_STATE_DISC ; ++iStart)
 			/* just search, no action */;
 		if(iStart == pBatch->nElem)
 			break; /* everything processed */
@@ -194,11 +194,11 @@ processBatchMultiRuleset(batch_t *pBatch)
 		for(i = iStart ; i < pBatch->nElem ; ++i) {
 			if(batchElemGetRuleset(pBatch, i) == currRuleset) {
 				/* for performance reasons, we copy only those members that we actually need */
-				snglRuleBatch.pElem[iNew].pUsrp = pBatch->pElem[i].pUsrp;
-				snglRuleBatch.pElem[iNew].state = pBatch->pElem[i].state;
+				snglRuleBatch.pElem[iNew].pMsg = pBatch->pElem[i].pMsg;
+				snglRuleBatch.eltState[iNew] = pBatch->eltState[i];
 				++iNew;
 				/* We indicate the element also as done, so it will not be processed again */
-				pBatch->pElem[i].state = BATCH_STATE_DISC;
+				pBatch->eltState[i] = BATCH_STATE_DISC;
 			} else {
 				bHaveUnprocessed = 1;
 			}
@@ -242,10 +242,10 @@ execSet(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 	struct var result;
 	DEFiRet;
 	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
-		if(   pBatch->pElem[i].state != BATCH_STATE_DISC
+		if(   pBatch->eltState[i] != BATCH_STATE_DISC
 		   && (active == NULL || active[i])) {
-			cnfexprEval(stmt->d.s_set.expr, &result, pBatch->pElem[i].pUsrp);
-			msgSetJSONFromVar((msg_t*)pBatch->pElem[i].pUsrp, stmt->d.s_set.varname,
+			cnfexprEval(stmt->d.s_set.expr, &result, pBatch->pElem[i].pMsg);
+			msgSetJSONFromVar(pBatch->pElem[i].pMsg, stmt->d.s_set.varname,
 					  &result);
 			varDelete(&result);
 		}
@@ -259,9 +259,9 @@ execUnset(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 	int i;
 	DEFiRet;
 	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
-		if(   pBatch->pElem[i].state != BATCH_STATE_DISC
+		if(   pBatch->eltState[i] != BATCH_STATE_DISC
 		   && (active == NULL || active[i])) {
-			msgUnsetJSON((msg_t*)pBatch->pElem[i].pUsrp, stmt->d.s_unset.varname);
+			msgUnsetJSON(pBatch->pElem[i].pMsg, stmt->d.s_unset.varname);
 		}
 	}
 	RETiRet;
@@ -277,9 +277,9 @@ execStop(batch_t *pBatch, sbool *active)
 	int i;
 	DEFiRet;
 	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
-		if(   pBatch->pElem[i].state != BATCH_STATE_DISC
+		if(   pBatch->eltState[i] != BATCH_STATE_DISC
 		   && (active == NULL || active[i])) {
-			pBatch->pElem[i].state = BATCH_STATE_DISC;
+			pBatch->eltState[i] = BATCH_STATE_DISC;
 		}
 	}
 	RETiRet;
@@ -297,31 +297,44 @@ execIf(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 	sbool *newAct;
 	int i;
 	sbool bRet;
+	sbool allInactive = 1;
 	DEFiRet;
 	newAct = newActive(pBatch);
-	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
-		if(pBatch->pElem[i].state == BATCH_STATE_DISC)
+	for(i = 0 ; i < batchNumMsgs(pBatch) ; ++i) {
+		if(*(pBatch->pbShutdownImmediate))
+			FINALIZE;
+		if(pBatch->eltState[i] == BATCH_STATE_DISC)
 			continue; /* will be ignored in any case */
 		if(active == NULL || active[i]) {
-			bRet = cnfexprEvalBool(stmt->d.s_if.expr,
-					       (msg_t*)(pBatch->pElem[i].pUsrp));
+			bRet = cnfexprEvalBool(stmt->d.s_if.expr, pBatch->pElem[i].pMsg);
+			allInactive = 0;
 		} else 
 			bRet = 0;
 		newAct[i] = bRet;
 		DBGPRINTF("batch: item %d: expr eval: %d\n", i, bRet);
 	}
 
+	if(allInactive) {
+		DBGPRINTF("execIf: all batch elements are inactive, holding execution\n");
+		freeActive(newAct);
+		FINALIZE;
+	}
+
 	if(stmt->d.s_if.t_then != NULL) {
 		scriptExec(stmt->d.s_if.t_then, pBatch, newAct);
 	}
 	if(stmt->d.s_if.t_else != NULL) {
-		for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate)
-		    ; ++i)
-			if(pBatch->pElem[i].state != BATCH_STATE_DISC)
+		for(i = 0 ; i < batchNumMsgs(pBatch) ; ++i) {
+			if(*(pBatch->pbShutdownImmediate))
+				FINALIZE;
+			if(pBatch->eltState[i] != BATCH_STATE_DISC
+			   && (active == NULL || active[i]))
 				newAct[i] = !newAct[i];
+			}
 		scriptExec(stmt->d.s_if.t_else, pBatch, newAct);
 	}
 	freeActive(newAct);
+finalize_it:
 	RETiRet;
 }
 
@@ -334,10 +347,12 @@ execPRIFILT(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 	int bRet;
 	int i;
 	newAct = newActive(pBatch);
-	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
-		if(pBatch->pElem[i].state == BATCH_STATE_DISC)
+	for(i = 0 ; i < batchNumMsgs(pBatch) ; ++i) {
+		if(*(pBatch->pbShutdownImmediate))
+			return;
+		if(pBatch->eltState[i] == BATCH_STATE_DISC)
 			continue; /* will be ignored in any case */
-		pMsg = (msg_t*)(pBatch->pElem[i].pUsrp);
+		pMsg = pBatch->pElem[i].pMsg;
 		if(active == NULL || active[i]) {
 			if( (stmt->d.s_prifilt.pmask[pMsg->iFacility] == TABLE_NOPRI) ||
 			   ((stmt->d.s_prifilt.pmask[pMsg->iFacility]
@@ -355,10 +370,13 @@ execPRIFILT(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 		scriptExec(stmt->d.s_prifilt.t_then, pBatch, newAct);
 	}
 	if(stmt->d.s_prifilt.t_else != NULL) {
-		for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate)
-		    ; ++i)
-			if(pBatch->pElem[i].state != BATCH_STATE_DISC)
+		for(i = 0 ; i < batchNumMsgs(pBatch) ; ++i) {
+			if(*(pBatch->pbShutdownImmediate))
+				return;
+			if(pBatch->eltState[i] != BATCH_STATE_DISC
+			   && (active == NULL || active[i]))
 				newAct[i] = !newAct[i];
+			}
 		scriptExec(stmt->d.s_prifilt.t_else, pBatch, newAct);
 	}
 	freeActive(newAct);
@@ -393,12 +411,12 @@ evalPROPFILT(struct cnfstmt *stmt, msg_t *pMsg)
 		break;
 	case FIOP_ISEQUAL:
 		if(rsCStrSzStrCmp(stmt->d.s_propfilt.pCSCompValue,
-				  pszPropVal, ustrlen(pszPropVal)) == 0)
+				  pszPropVal, propLen) == 0)
 			bRet = 1; /* process message! */
 		break;
 	case FIOP_STARTSWITH:
 		if(rsCStrSzStrStartsWithCStr(stmt->d.s_propfilt.pCSCompValue,
-				  pszPropVal, ustrlen(pszPropVal)) == 0)
+				  pszPropVal, propLen) == 0)
 			bRet = 1; /* process message! */
 		break;
 	case FIOP_REGEX:
@@ -462,11 +480,13 @@ execPROPFILT(struct cnfstmt *stmt, batch_t *pBatch, sbool *active)
 	sbool bRet;
 	int i;
 	thenAct = newActive(pBatch);
-	for(i = 0 ; i < batchNumMsgs(pBatch) && !*(pBatch->pbShutdownImmediate) ; ++i) {
-		if(pBatch->pElem[i].state == BATCH_STATE_DISC)
+	for(i = 0 ; i < batchNumMsgs(pBatch) ; ++i) {
+		if(*(pBatch->pbShutdownImmediate))
+			return;
+		if(pBatch->eltState[i] == BATCH_STATE_DISC)
 			continue; /* will be ignored in any case */
 		if(active == NULL || active[i]) {
-			bRet = evalPROPFILT(stmt, (msg_t*)(pBatch->pElem[i].pUsrp));
+			bRet = evalPROPFILT(stmt, pBatch->pElem[i].pMsg);
 		} else 
 			bRet = 0;
 		thenAct[i] = bRet;
@@ -494,7 +514,11 @@ scriptExec(struct cnfstmt *root, batch_t *pBatch, sbool *active)
 	struct cnfstmt *stmt;
 
 	for(stmt = root ; stmt != NULL ; stmt = stmt->next) {
-dbgprintf("RRRR: scriptExec: batch of %d elements, active %p, stmt %p, nodetype %u\n", batchNumMsgs(pBatch), active, stmt, stmt->nodetype);
+		if(Debug) {
+			dbgprintf("scriptExec: batch of %d elements, active %p, active[0]:%d\n",
+				  batchNumMsgs(pBatch), active, (active == NULL ? 1 : active[0]));
+			cnfstmtPrintOnly(stmt, 2, 0);
+		}
 		switch(stmt->nodetype) {
 		case S_NOP:
 			break;
@@ -511,7 +535,6 @@ dbgprintf("RRRR: scriptExec: batch of %d elements, active %p, stmt %p, nodetype
 			execUnset(stmt, pBatch, active);
 			break;
 		case S_CALL:
-			DBGPRINTF("calling ruleset\n"); // TODO: add Name
 			scriptExec(stmt->d.s_call.stmt, pBatch, active);
 			break;
 		case S_IF:
@@ -719,7 +742,7 @@ CODESTARTobjDestruct(ruleset)
 		parser.DestructParserList(&pThis->pParserLst);
 	}
 	free(pThis->pszName);
-	cnfstmtDestruct(pThis->root);
+	cnfstmtDestructLst(pThis->root);
 ENDobjDestruct(ruleset)
 
 
@@ -845,7 +868,7 @@ doRulesetCreateQueue(rsconf_t *conf, int *pNewVal)
 
 	rsname = (conf->rulesets.pCurr->pszName == NULL) ? (uchar*) "[ruleset]" : conf->rulesets.pCurr->pszName;
 	DBGPRINTF("adding a ruleset-specific \"main\" queue for ruleset '%s'\n", rsname);
-	CHKiRet(createMainQueue(&conf->rulesets.pCurr->pQueue, rsname));
+	CHKiRet(createMainQueue(&conf->rulesets.pCurr->pQueue, rsname, NULL));
 
 finalize_it:
 	RETiRet;
@@ -905,6 +928,7 @@ rsRetVal
 rulesetProcessCnf(struct cnfobj *o)
 {
 	struct cnfparamvals *pvals;
+	struct cnfparamvals *queueParams;
 	rsRetVal localRet;
 	uchar *rsName = NULL;
 	uchar *parserName;
@@ -912,6 +936,7 @@ rulesetProcessCnf(struct cnfobj *o)
 	ruleset_t *pRuleset;
 	struct cnfarray *ar;
 	int i;
+	uchar *rsname;
 	DEFiRet;
 
 	pvals = nvlstGetParams(o->nvlst, &rspblk, NULL);
@@ -927,7 +952,7 @@ rulesetProcessCnf(struct cnfobj *o)
 		errmsg.LogError(0, RS_RET_RULESET_EXISTS,
 			"error: ruleset '%s' specified more than once",
 			rsName);
-		cnfstmtDestruct(o->script);
+		cnfstmtDestructLst(o->script);
 		ABORT_FINALIZE(RS_RET_RULESET_EXISTS);
 	} else if(localRet != RS_RET_NOT_FOUND) {
 		ABORT_FINALIZE(localRet);
@@ -939,14 +964,21 @@ rulesetProcessCnf(struct cnfobj *o)
 
 	/* we have only two params, so we do NOT do the usual param loop */
 	parserIdx = cnfparamGetIdx(&rspblk, "parser");
-	if(parserIdx == -1  || !pvals[parserIdx].bUsed)
-		FINALIZE;
+	if(parserIdx != -1  && pvals[parserIdx].bUsed) {
+		ar = pvals[parserIdx].val.d.ar;
+		for(i = 0 ; i <  ar->nmemb ; ++i) {
+			parserName = (uchar*)es_str2cstr(ar->arr[i], NULL);
+			doRulesetAddParser(pRuleset, parserName);
+			free(parserName);
+		}
+	}
 
-	ar = pvals[parserIdx].val.d.ar;
-	for(i = 0 ; i <  ar->nmemb ; ++i) {
-		parserName = (uchar*)es_str2cstr(ar->arr[i], NULL);
-		doRulesetAddParser(pRuleset, parserName);
-		free(parserName);
+	/* pick up ruleset queue parameters */
+	qqueueDoCnfParams(o->nvlst, &queueParams);
+	if(queueCnfParamsSet(queueParams)) {
+		rsname = (pRuleset->pszName == NULL) ? (uchar*) "[ruleset]" : pRuleset->pszName;
+		DBGPRINTF("adding a ruleset-specific \"main\" queue for ruleset '%s'\n", rsname);
+		CHKiRet(createMainQueue(&pRuleset->pQueue, rsname, queueParams));
 	}
 
 finalize_it:
