@@ -10,7 +10,7 @@ source $srcdir/diag.sh init
 source $srcdir/diag.sh startup discard.conf
 # 20000 messages should be enough - the disk test is slow enough ;)
 sleep 4
-source $srcdir/diag.sh tcpflood 127.0.0.1 13514 1 10 1
+source $srcdir/diag.sh tcpflood -m10 -i1
 source $srcdir/diag.sh shutdown-when-empty # shut down rsyslogd when done processing messages
 source $srcdir/diag.sh wait-shutdown
 source $srcdir/diag.sh seq-check 2 10
