@@ -1,4 +1,4 @@
-echo TEST: proprepltest.sh - various tests for the property replacer
+echo \[proprepltest.sh\]: various tests for the property replacer
 source $srcdir/diag.sh init
 source $srcdir/diag.sh nettester rfctag udp
 source $srcdir/diag.sh nettester rfctag tcp
