@@ -258,7 +258,7 @@ int openConnections(void)
 		return setupUDP();
 
 	if(bShowProgress)
-		write(1, "      open connections", sizeof("      open connections")-1);
+		if(write(1, "      open connections", sizeof("      open connections")-1)){}
 #	ifdef ENABLE_GNUTLS
 	sessArray = calloc(numConnections, sizeof(gnutls_session_t));
 #	endif
@@ -278,7 +278,7 @@ int openConnections(void)
 	}
 	if(bShowProgress) {
 		lenMsg = sprintf(msgBuf, "\r%5.5d open connections\n", i);
-		write(1, msgBuf, lenMsg);
+		if(write(1, msgBuf, lenMsg)) {}
 	}
 
 	return 0;
@@ -303,12 +303,12 @@ void closeConnections(void)
 		return;
 
 	if(bShowProgress)
-		write(1, "      close connections", sizeof("      close connections")-1);
+		if(write(1, "      close connections", sizeof("      close connections")-1)){}
 	for(i = 0 ; i < numConnections ; ++i) {
 		if(i % 10 == 0) {
 			if(bShowProgress) {
 				lenMsg = sprintf(msgBuf, "\r%5.5d", i);
-				write(1, msgBuf, lenMsg);
+				if(write(1, msgBuf, lenMsg)){}
 			}
 		}
 		if(sockArray[i] != -1) {
@@ -325,7 +325,7 @@ void closeConnections(void)
 	}
 	if(bShowProgress) {
 		lenMsg = sprintf(msgBuf, "\r%5.5d close connections\n", i);
-		write(1, msgBuf, lenMsg);
+		if(write(1, msgBuf, lenMsg)){}
 	}
 
 }
@@ -347,7 +347,7 @@ genMsg(char *buf, size_t maxBuf, int *pLenBuf, struct instdata *inst)
 		/* get message from file */
 		do {
 			done = 1;
-			*pLenBuf = fread(buf, 1, 1024, dataFP);
+			*pLenBuf = fread(buf, 1, MAX_EXTRADATA_LEN + 1024, dataFP);
 			if(*pLenBuf == 0) {
 				if(--numFileIterations > 0)  {
 					rewind(dataFP);
@@ -429,6 +429,8 @@ int sendMessages(struct instdata *inst)
 			}
 		}
 		genMsg(buf, sizeof(buf), &lenBuf, inst); /* generate the message to send according to params */
+		if(lenBuf == 0)
+			break;	/* terminate when no message could be generated */
 		if(transport == TP_TCP) {
 			if(sockArray[socknum] == -1) {
 				/* connection was dropped, need to re-establish */
