@@ -11,6 +11,9 @@ source $srcdir/diag.sh init
 source $srcdir/diag.sh startup threadingmq.conf
 source $srcdir/diag.sh injectmsg 0 100000
 source $srcdir/diag.sh shutdown-when-empty # shut down rsyslogd when done processing messages
+# we give an extra seconds for things to settle, especially
+# important on slower test machines
+./msleep 1000
 source $srcdir/diag.sh wait-shutdown
 source $srcdir/diag.sh seq-check 0 99999
 source $srcdir/diag.sh exit
