@@ -55,7 +55,8 @@ int read_pid (char *pidfile)
 
   if (!(f=fopen(pidfile,"r")))
     return 0;
-  fscanf(f,"%d", &pid);
+  if(fscanf(f,"%d", &pid) != 1)
+  	pid = 0;
   fclose(f);
   return pid;
 }
@@ -113,7 +114,8 @@ int write_pid (char *pidfile)
 
 #if HAVE_FLOCK
   if (flock(fd, LOCK_EX|LOCK_NB) == -1) {
-      fscanf(f, "%d", &pid);
+      if(fscanf(f, "%d", &pid) != 1)
+      	pid = 0;
       fclose(f);
       printf("Can't lock, lock is held by pid %d.\n", pid);
       return 0;
