@@ -51,7 +51,6 @@
 #include <stddef.h>
 #include <ctype.h>
 #include <limits.h>
-#define GNU_SOURCE
 #include <string.h>
 #include <stdarg.h>
 #include <time.h>
@@ -2034,6 +2033,8 @@ int realMain(int argc, char **argv)
 		ourConf->globals.bErrMsgToStderr = 0;
 	}
 
+	sd_notify(0, "READY=1");
+
 	mainloop();
 
 	/* do any de-init's that need to be done AFTER this comment */
